program sorosportpelda;

// Soros port olvasása és írása a SynaSer unit használatával (Ararat Synapse)
// Írta: Péter Gábor (ptrg@freemail.hu)

// A fordításhoz szükség van a következő csomagra: laz_synapse (nálam V40.1 [r192])
// A laz_synapse letölthető innen: http://synapse.ararat.cz/doku.php/download
// Információk és súgó a következő helyeken:
// http://synapse.ararat.cz/
// http://synapse.ararat.cz/doc/help/
// http://synapse.ararat.cz/doc/help/synaser.html
// http://synapse.ararat.cz/doc/help/synaser.TBlockSerial.html

// FONTOS: Unix típusú rendszereken csak rendszergazdai jogokkal működik (pl.: Linux, BSD)

uses
  SynaSer, // soros port kezelése
  SysUtils, // ez a kivételkezelés miatt szerepel itt
  CRT; // ez csak a billentyűzettel történő kilépés miatt szerepel itt

type
  TAllapotValtozasKezelo = class(TObject) // az állapotváltozásokat kezelő eljárást tartalmazó objektum typusa
    procedure VisszaHivas(Sender: TObject; Reason: THookSerialReason; const Value: string); // az állapotváltozásokat kezelő eljárás
  end;

var
  SorosPort: TBlockSerial; // a soros port objektum
  AllapotValtozasKezelo: TAllapotValtozasKezelo; // az állapotváltozásokat kezelő eljárást tartalmazó objektum
  B: Byte; // ebbe kerül majd a beolvasott adat
  C: Char; // a billentyűzetről kapott karakter

procedure TAllapotValtozasKezelo.VisszaHivas(Sender: TObject; Reason: THookSerialReason; const Value: string); // az állapotváltozásokat kezelő eljárás
begin
  if (Reason = HR_CanRead) then // van mit olvasni
    WriteLn('            - OLVASNIVALÓ VAN')
  else
    if (Reason = HR_CanWrite) then // lehet írni a portra
      WriteLn('            - ÍRHATÓ');
end;

begin

  AllapotValtozasKezelo := TAllapotValtozasKezelo.Create; // az állapotváltozásokat kezelő eljárást tartalmazó objektum létrehozása

  SorosPort := TBlockSerial.Create; // a soros port objektum létrehozása
  SorosPort.OnStatus := @AllapotValtozasKezelo.VisszaHivas; // az állapotváltozásokat kezelő eljárás beállítása

  try // befejezés (finally)

    try  // kivételkezelés (except)

      SorosPort.RaiseExcept := True; // a hibák kivételt okozzanak vagy sem (hibakód legyen beállítva), ez a példaprogram mindkét beállításra fel van készítve

      SorosPort.Connect('COM1'); // a port amelyet használni fogunk
      SorosPort.Config(9600, 8, 'N', 0, False, False); // beállítások - ez a sor csak akkor kell ha nem a rendszer alapbeállításait szeretnénk használni

      while SorosPort.LastError = 0 do // helybenjárás amíg nincs hiba (lásd fentebb: RaiseExcept)
      begin

        // Itt most puffer nélküli megoldás van, de puffer használata is megoldható a CanReadEx, a ReadBufferEx és társaik által
        if SorosPort.CanRead(1000) then // ellenőrizzük, hogy van-e olvasnivaló, max 1000ms (1mp) várakozás ha nincs
        begin
          B := SorosPort.RecVByte(1000); // egy bájt olvasása a portról, max 1000ms (1mp) várakozás ha nem sikerül
          WriteLn('>> ', B: 3, ' ', Chr(B), '   '); // a kapott bájt kiírása számként és karakterként a szabványos kimenetre
        end;

        if KeyPressed then // ha billentyűleütés történt
        begin
          C := ReadKey; // karakter beolvasása a billentyűzetről
          case C of
            #27: // ESC esetén
              Break;// megszakítjuk a helybenjárást
            else // minden más esetben
              if SorosPort.CanWrite(1000) then // ellenőrizzük, hogy írhatunk-e a port-ra, max 1000ms (1mp) újratesztelés ha nem
              begin
                SorosPort.SendByte(Ord(C)); // a karakter küldése a portra
                WriteLn('   ', Ord(C): 3, ' ', C, ' >>'); // a karakter kódjának és képének kiírása a szabványos kimenetre
              end;
          end;
        end;

      end;

      if SorosPort.LastError <> 0 then // ha hiba volt
        WriteLn('Hiba: ', SorosPort.LastErrorDesc, ' (', SorosPort.LastError, ')'); // a hiba kiírása

    except // ha kivétel keletkezett
      WriteLn('Kivétel: ', SorosPort.LastErrorDesc, ' (', SorosPort.LastError, ')'); // a hiba kiírása

    end;

  finally // befejezés
    SorosPort.Free; // felszabadítás

  end;

end.
